"""
Serveur HTTP rudimentaire, destiné à la mise en place d'interfaces graphiques html.
  affecter_commande : pour associer une requête HTTP à une fonction python
  servir            : pour démarrer le serveur
  stop              : pour arrêter le serveur
Auteur : M. Bordonaro
"""

import socket

ucd = {}
continuer = True

def affecter_commande( url : str, commande ):
    """
      l'url reçue par le serveur étant découpée en CIBLE?PARAM,
      renvoie le résultat de l'appel de 
      commande ( CIBLE : str, PARAM ) -> str
    """
    ucd[url] = commande

def stop_tp( url, commande):
    """
      provoque l'arrêt du serveur en répondant un message court
    """
    global continuer
    continuer = False
    return "<!doctype html><html><head><title>Arrêt</title><meta charset='utf-8'></head><body>Serveur arrêté.</body></html>"
    
def stop():
    """
      provoque l'arrêt du serveur après la fin du traitement de la requête en cours
    """
    global continuer
    continuer = False
    
def servir(ip = '127.0.0.1', port = 5080, max_conn = -1):
    """
       appel bloquant démarrant un serveur sur le port 'port' de l'interface 'ip'
       
       si max_conn > 0, le serveur s'arrête après avoir répondu à max_conn requêtes
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind((ip, port))
    s.listen()

    print("Démarrage du serveur http://"+ip+":"+str(port))

    while (continuer and max_conn != 0):
        max_conn = min(max_conn, max (max_conn - 1 ,0))

        t,(h,p) = s.accept()
        
        t.settimeout(0.05)

        try:
            req = t.recv(2048)
        except socket.timeout:
            pass
#            print ('Timeout !')
        except:
            pass
#            print ('Autre...')
        else:

            t.shutdown(socket.SHUT_RD)
            
            query = req.split(b'\r\n',1)[0]
            elts = query.split(b" ")

            if elts[0] == b'GET': 
                url = elts[1].split(b"?",1)
                comm = pourcent_dec_get((url[0]))
                if len(url) == 1:
                    param = ""
                else:
                    param = pourcent_dec_get(url[1])
                params = extraire(param)
                if comm in ucd:
                    bb = bytes( ucd[comm]( comm, params),'utf-8')
                    bb = b'HTTP/1.1 200 OK\r\nContent-Type: text/html; charset=utf-8\r\nConnection: close\r\nContent-Length: ' + bytes(str(len(bb)),"utf-8") + b'\r\n\r\n' + bb
                    t.sendall(bb)
#                    print ("Sent !") 
                else:
                    t.sendall(bytes('HTTP/1.1 404 Not Found\r\nContent-Type: text/plain; charset=utf-8\r\nConnection: close\r\¬Content-Length:17\r\n\r\nPas trouvé !\r\n\r\n','utf-8'))
#                    print ("Sent !")
            else:
                t.sendall(bytes('HTTP/1.1 400 Bad Request\r\nContent-Type: text/plain; charset=utf-8\r\nConnection: close\r\nContent-Length: 26\r\n\r\nRequête mal formée !\r\n\r\n','utf-8'))
#                print ("Sent !")

        try:
            t.shutdown(socket.SHUT_WR)
#            print ("Bye !")
            t.close()
#            print ("Bye !")
        except:
            pass

#    s.shutdown(socket.SHUT_RDWR)
    s.close()


def pourcent_dec_get( burl : bytes):
    """
      pratique un url->utf-8, mais =, & et % restent %-encodés
    """
    res = b''
    k=0
    while k < len(burl):
        if burl[k] == b'%'[0]:
            if int(burl[k+1:k+3],16) in [0x3D, 0x26, 0x25]:
                #c'est '&' ou '=' ou '%', on le garde pour la fin
                res = res + burl[k:k+3]
            else:
                 #on décode, sauf 3D et 26 et 25
                res = res + bytes([int(burl[k+1:k+3],16)])
            k = k+3
        else:
            res = res + bytes([burl[k]])
            k = k + 1
    return (str(res,"utf-8"))

def extraire( url : str):
    """
       on renvoie un dico avec tous les paramètres passés pas url
    """
    res = {}
    lc = url.split('&')
    for aff in lc:
        if '=' in aff:
            c,v = aff.split('=',1)
        else:
            c = aff
            v=''
        c = c.replace("%3D","=").replace("%3d","=").replace("%26","&").replace("%25","%")
        v = v.replace("%3D","=").replace("%3d","=").replace("%26","&").replace("%25","%")
        res[c] = v

    return res

#def extraire_param( url : str, param: str):
#    """
#      on extrait un paramètre d'une url, puis on %-décode seuement =, & et %
#    """
#    res = ""
#    ideb = url.find(param)
#    if ideb != - 1:
#        ifin = url[ideb+len(param)+1:].find("&")
#        if ifin != -1:
#            res = url[ideb+len(param)+1:][:ifin]
#        else:
#            res = url[ideb+len(param)+1:]
#    return res.replace("%3D","=").replace("%3d","=").replace("%26","&").replace("%25","%")


#def remplir_page( page: str, param: dict):
#    """
#    on place des valeurs dans une page
#    """
#    for clef in param.keys():
#        page = page.replace(clef, param[clef])
#    return page
